@@ -1,11 +1,12 @@
 import asyncio
 import logging
+import os
 import traceback
 import typing
 from random import Random
 from typing import TYPE_CHECKING
 from dataclasses import dataclass
-    
+import urllib.parse
 
 import colorama
 import Utils
@@ -14,6 +15,8 @@ from CommonClient import ClientCommandProcessor, CommonContext, server_loop, gui
 apname = Utils.instance_name if Utils.instance_name else "Archipelago"
 from pathlib import Path
 
+from worlds.poe import Items
+
 from .poeClient.fileHelper import load_settings, save_settings, find_possible_client_txt_path
 from .poeClient import main as poe_main
 from .poeClient import gggAPI
@@ -141,6 +144,38 @@ class PathOfExileCommandProcessor(ClientCommandProcessor):
         tts.run_tts_tasks()
         return True
 
+    def _cmd_whisper_updates(self, enable: bool | str | None = None) -> bool:
+        """Enable or disable whispering item updates to the in game chat."""
+        if enable is None:
+            whisper_enabled_implied = not self.ctx.game_options.get("whisper_updates", False)
+            self.output(f"Turning whisper updates {'on' if whisper_enabled_implied else 'off'}")
+            enable = whisper_enabled_implied
+        if isinstance(enable, str):
+            lowered = enable.lower()
+            if lowered in {"true", "1", "yes", "y", "on"}:
+                enable_bool = True
+            elif lowered in {"false", "0", "no", "n", "off"}:
+                enable_bool = False
+            else:
+                self.output("ERROR: Please provide a valid boolean value for enabling whisper updates (True/False).")
+                return False
+        elif isinstance(enable, bool):
+            enable_bool = enable
+        else:
+            self.output("ERROR: Please provide a valid boolean value for enabling whisper updates (True/False).")
+            return False
+
+        self.ctx.game_options["whisper_updates"] = enable_bool
+        self.ctx.whisper_updates_enabled = enable_bool
+        self.ctx.update_settings()
+
+        if enable_bool:
+            self.output("Whisper updates enabled.")
+        else:
+            self.output("Whisper updates disabled.")
+        return True
+
+
     def _cmd_poe_auth(self) -> bool:        
         """Authenticate with Path of Exile's OAuth2 service."""
         def on_complete(task):
@@ -220,13 +255,23 @@ class PathOfExileCommandProcessor(ClientCommandProcessor):
         #required
         self.logger.info(f"Path of Exile apworld version: {POE_VERSION}")
         self.output(f"Path of Exile apworld version: {POE_VERSION}")
+
+        if not self.ctx.slot_data:
+            self.output(f"_________________________________________________________________________________\n"
+                        f"Connect to a multiworld server before starting the Path of Exile client!\n"
+                        f"Use either the GUI or the '/connect' command to connect.\n"
+                        f"_________________________________________________________________________________")
+            return False
+
         if not self.ctx.client_text_path:
             possible_path = find_possible_client_txt_path()
             if possible_path:
                 self.ctx.client_text_path = possible_path
-                self.output(f"Using default a possible client text path located here: {self.ctx.client_text_path},\n "
-                            f"THIS MAY NOT BE THE CORRECT PATH, please verify it is correct, and change it otherwise "
-                            f"using the 'set_client_text_path <path>' command.")
+                self.output(f"_________________________________________________________________________________\n"
+                            f"Using a possible client text path located here: {self.ctx.client_text_path},\n "
+                            f"THIS MAY NOT BE THE CORRECT PATH, please verify it is correct, and change it if wrong "
+                            f"using the 'set_client_text_path <path>' command.\n"
+                            f"_________________________________________________________________________________")
             else:
                 self.output("Please set the client text path using the 'set_client_text_path <path>' command.")
                 return False
@@ -287,7 +332,7 @@ class PathOfExileCommandProcessor(ClientCommandProcessor):
 def handle_task_errors(task: asyncio.Task, ctx: "PathOfExileContext", cmdprocessor: PathOfExileCommandProcessor):
     """Handle errors in the task."""
     try:
-        task.result()  # Will raise if the task failed
+         task.result()  # Will raise if the task failed
     except Exception as e:
         cmdprocessor.output(f"ERROR, the client borked: {e}")
 
@@ -312,6 +357,9 @@ class FilterOptions:
     #    sfx_enabled: bool = True  # Whether to play random sound effects for item pickups (trap)
     
 class PathOfExileContext(CommonContext):
+    # THESE ARE ALL STATIC.
+    # but I guess that's ok, because we only have one client per game.
+
     game = "Path of Exile"
     command_processor = PathOfExileCommandProcessor
     items_handling = 0b111
@@ -323,11 +371,15 @@ class PathOfExileContext(CommonContext):
     passives_used: int = 0
     passives_available: int = 0
 
+    previously_received_item_ids: list[int] = []
     character_name: str = ""
     client_text_path: Path = ""
     base_item_filter: str = ""
     poe_doc_path: str = ""
     generated_version: str = ""
+    whisper_updates_enabled: bool = True
+    loaded_client_settings: bool = False
+    text_to_send: list[tuple[str, bool]] = [] # list of (message, prepend_char_name) tuples to send to in-game chat
     slot_data = {}
     game_options = {}
     client_options = {}
@@ -344,7 +396,7 @@ class PathOfExileContext(CommonContext):
         super().__init__(server_address, password)
         self.ready_callback = ready_callback
         self.error_callback = error_callback
-        self.username = slot_name
+        self.username = urllib.parse.urlparse(server_address).username
         if self.ready_callback:
             from kivy.clock import Clock
             Clock.schedule_once(self.ready_callback, 0.1)
@@ -375,17 +427,17 @@ class PathOfExileContext(CommonContext):
 
             if self.generated_version != POE_VERSION:
                 if self.generated_version in BACKWARDS_COMPATIBLE_VERSIONS:
-                    log = (f"Connected to multiworld generated with different version: {self.generated_version}, "+
+                    log = (f"Connected to a multiworld generated with a different version: {self.generated_version}, "+
                            f"running version {POE_VERSION}. This is marked as backwards compatible, and should be OK.")
                     self.logger.info(log)
                     self.command_processor.output(self=self.command_processor, text=log)
                 else:
-                    log = (f"-----------------------------------------------------------------------------------------\n"+
+                    log = (f"_________________________________________________________________________________\n"+
                            f"Server generated with unsupported version!\n"+
                            f"Server:{self.generated_version}\n"+
                            f"Client:{POE_VERSION}\n"+
                            f"This may cause issues!!!\n"+
-                           f"-----------------------------------------------------------------------------------------")
+                           f"_________________________________________________________________________________")
                     self.logger.warning(log)
                     self.command_processor.output(self=self.command_processor, text=log)
 
@@ -414,6 +466,9 @@ class PathOfExileContext(CommonContext):
                         self.character_name = settings.get("last_char", self.character_name)
                         self.base_item_filter = settings.get("base_item_filter", self.base_item_filter)
                         self.poe_doc_path = settings.get("poe_doc_path", self.poe_doc_path)
+                        self.whisper_updates_enabled = settings.get("whisper_updates", True)
+                        self.previously_received_item_ids = settings.get("already_received_items", [])
+                        self.loaded_client_settings = True
                         logger.debug(f"[DEBUG] Loaded settings: {settings}")
                     if self.poe_doc_path:
                         itemFilter.set_poe_doc_path(self.poe_doc_path)
@@ -424,13 +479,13 @@ class PathOfExileContext(CommonContext):
                 self.command_processor.output(self=self.command_processor, text=msg)
             def load_client_settings(task=None):
                 if not self.seed_name:
-                    self.logger.info("ERROR: No seed name found in RoomInfo!!!!! STILL IDK WHY.")
+                    self.logger.info("[DEBUG]: No seed name found in RoomInfo!!!!! STILL IDK WHY.")
                 task = asyncio.create_task(load_settings(self))
                 task.add_done_callback(injest_load_client_settings)
 
             if not self.seed_name:
 
-                self.logger.info("ERROR: No seed name found in RoomInfo. IDK WHY.")
+                self.logger.info("[DEBUG]: No seed name found in RoomInfo. IDK WHY.")
                 asyncio.create_task(asyncio.sleep(0.2)).add_done_callback(load_client_settings)
 
             else:
@@ -441,6 +496,42 @@ class PathOfExileContext(CommonContext):
 
             # self.command_processor.output(self=self, text=f"[color=green]{msg}[/color]") #TODO: color in GUI
 
+        if cmd == 'ReceivedItems':
+            self.send_received_item_updates()
+
+    def send_received_item_updates(self):
+        if not self.loaded_client_settings:
+            return # don't send updates until settings are loaded, because we would be duplicating messages
+        received_ids: list[int] = [item.item for item in self.items_received]
+        new_items: list[str] = []
+        added_ids = set()
+        self.logger.info(f"[DEBUG]: Received {len(received_ids)} items")
+        for network_item in self.items_received:
+            # Newly-obtained items
+            is_more_than_one = Items.item_table[network_item.item].get("count", 0)
+
+            if (not network_item.item in self.previously_received_item_ids or is_more_than_one) and network_item.item not in added_ids:
+                added_ids.add(network_item.item)
+
+                # handle items with more than 1 count
+                if is_more_than_one:
+                    count_of_items_already_received = self.previously_received_item_ids.count(network_item.item)
+                    count_of_sent_received = [i.item for i in self.items_received].count(network_item.item)
+                    count_of_item = count_of_sent_received - count_of_items_already_received
+                    if count_of_item <= 0:
+                        continue
+                    elif count_of_item > 1:
+                        new_items.append(f"{count_of_item}x {str(self.item_names.lookup_in_game(network_item.item))}")
+                    else:# count_of_item == 1:
+                        new_items.append(str(self.item_names.lookup_in_game(network_item.item)))
+                    continue
+
+                new_items.append(str(self.item_names.lookup_in_game(network_item.item)))
+        if self.whisper_updates_enabled and new_items:
+            self.text_to_send.append(("You received new items! : " + ", ".join(new_items), True))
+        self.previously_received_item_ids = received_ids
+        if self.loaded_client_settings:
+            self.update_settings()
 
     def update_settings(self):
         """Update a setting and save it to the settings file."""
@@ -476,7 +567,7 @@ class PathOfExileContext(CommonContext):
 #     await ctx.shutdown()
 
 
-def launch(server_address: str = None, slot_name: str = None, password: str = None, ready_callback=None, error_callback=None):
+def launch(server_address: str = None, password: str = None, ready_callback=None, error_callback=None):
     """
     Launch the client
     """
@@ -484,7 +575,7 @@ def launch(server_address: str = None, slot_name: str = None, password: str = No
     logger = logging.getLogger("PathOfExileClient")
 
     async def main():
-        ctx = PathOfExileContext(server_address, slot_name, password, ready_callback, error_callback)
+        ctx = PathOfExileContext(server_address, password, ready_callback, error_callback)
         if ctx._can_takeover_existing_gui():
             await ctx._takeover_existing_gui() 
         else:
@@ -521,9 +612,9 @@ def launch(server_address: str = None, slot_name: str = None, password: str = No
             error_callback()
 
 
-def main(server_address: str = None, slot_name: str = None, password: str = None, ready_callback=None, error_callback=None):
+def main(server_address: str = None, password: str = None, ready_callback=None, error_callback=None):
     """Main entry point for integration with MultiWorld system"""
-    launch(server_address, slot_name, password, ready_callback, error_callback)
+    launch(server_address, password, ready_callback, error_callback)
 
 
 
