@@ -74,7 +74,8 @@ async def chat_commands_callback(ctx: "PathOfExileContext", line: str):
     char_name, message = get_char_name_and_message_from_line(line)
     message = message.lower()
 
-    if "!ap char" in message:
+    if (any(cmd in message for cmd in ("!ap char", "!ap character", "!ap setchar", "!ap setcharacter", "!ap_char", "!apchar"))
+        and not "`!ap char`" in message):  # prevent accidental triggers
         _random_string = random.randbytes(8).hex()
         await inputHelper.send_poe_text(f"char_{_random_string}") # don't know the char name yet, so we can't whisper.
     if "char_" in message:
@@ -90,75 +91,77 @@ async def chat_commands_callback(ctx: "PathOfExileContext", line: str):
 
     if not char_name == ctx.character_name:
         return
-    item_ids = [item.item for item in ctx.items_received]
+    received_item_ids = [item.item for item in ctx.items_received]
     
-    if "!help" in message or "!commands" in message:
+    if any(cmd in message for cmd in ("!help", "!commands", "!cmds")):
             help_message = """!ap char - Set your character | !deathlink | !goal | !passive or !p | !usable skill gems - by level | !usable support gems | !usable utility gems | !usable gems | !main gems | !support gems | !utility gems | !all gems or !gems | !gear | !weapons | !armor | !links | !flasks | !ascendancy | !help | Note: use @yourname followed a command."""
             await split_send_message(ctx, help_message)
             
     if "!main gems" in message:
         # Get all main skill gem items in item_ids
-        gems = [item for item in Items.get_main_skill_gem_items() if item["id"] in item_ids] + [i for i in Items.get_by_category("GemModifier") if i["id"] in item_ids]
+        gems = [item for item in Items.get_main_skill_gem_items() if item["id"] in received_item_ids] + [i for i in Items.get_by_category("GemModifier") if i["id"] in received_item_ids]
         # sort by required level
         gems.sort(key=lambda x: x.get("requiredLevel", 0))  # Sort by required level
         await split_send_message(ctx,', '.join(gem['name'] for gem in gems))
 
     if "!support gems" in message:
         # Get all support gem items in item_ids
-        gems = [item for item in Items.get_support_gem_items() if item["id"] in item_ids]
+        gems = [item for item in Items.get_support_gem_items() if item["id"] in received_item_ids]
         gems.sort(key=lambda x: x.get("requiredLevel", 0))  # Sort by required level
         await split_send_message(ctx,', '.join(gem['name'] for gem in gems))
 
     if "!utility gems" in message:
         # Get all utility gem items in item_ids
-        gems = [item for item in Items.get_utility_skill_gem_items() if item["id"] in item_ids]
+        gems = [item for item in Items.get_utility_skill_gem_items() if item["id"] in received_item_ids]
         gems.sort(key=lambda x: x.get("requiredLevel", 0))  # Sort by required level
         await split_send_message(ctx,', '.join(gem['name'] for gem in gems))
         
-    if "!all gems" in message or "!gems" in message:
+    if any(cmd in message for cmd in ("!gems", "!all gems")):
         # Get all gem items in item_ids
-        gems = [item for item in Items.get_all_gems() if item["id"] in item_ids] + [i for i in Items.get_by_category("GemModifier") if i["id"] in item_ids]
+        gems = [item for item in Items.get_all_gems() if item["id"] in received_item_ids] + [i for i in Items.get_by_category("GemModifier") if i["id"] in received_item_ids]
         gems.sort(key=lambda x: x.get("requiredLevel", 0))  # Sort by required level
         await split_send_message(ctx,', '.join(gem['name'] for gem in gems))
 
     if "!usable skill gems" in message:
         # Get all usable skill gems in item_ids
-        usable_gems = [item for item in Items.get_main_skill_gems_by_required_level(0, ctx.last_character_level) if item["id"] in item_ids]
+        usable_gems = [item for item in Items.get_main_skill_gems_by_required_level(0, ctx.last_character_level) if item["id"] in received_item_ids]
         usable_gems.sort(key=lambda x: x.get("requiredLevel", 0), reverse=True)  # Sort by required level descending
         await split_send_message(ctx,', '.join(gem['name'] for gem in usable_gems))
 
     if "!usable support gems" in message:
         # Get all usable skill gems in item_ids
-        usable_gems = [item for item in Items.get_support_gems_by_required_level(0, ctx.last_character_level) if item["id"] in item_ids]
+        usable_gems = [item for item in Items.get_support_gems_by_required_level(0, ctx.last_character_level) if item["id"] in received_item_ids]
         usable_gems.sort(key=lambda x: x.get("requiredLevel", 0), reverse=True)  # Sort by required level descending
         await split_send_message(ctx,', '.join(gem['name'] for gem in usable_gems))
 
     if "!usable utility gems" in message:
         # Get all usable utility gems in item_ids
-        usable_gems = [item for item in Items.get_utility_skill_gems_by_required_level(0, ctx.last_character_level) if item["id"] in item_ids]
+        usable_gems = [item for item in Items.get_utility_skill_gems_by_required_level(0, ctx.last_character_level) if item["id"] in received_item_ids]
         usable_gems.sort(key=lambda x: x.get("requiredLevel", 0), reverse=True)  # Sort by required level descending
         await split_send_message(ctx,', '.join(gem['name'] for gem in usable_gems))
 
     if "!usable gems" in message:
         # Get all usable gems in item_ids
-        usable_gems = [item for item in Items.get_all_gems_by_required_level(0, ctx.last_character_level) if item["id"] in item_ids]
+        usable_gems = [item for item in Items.get_all_gems_by_required_level(0, ctx.last_character_level) if item["id"] in received_item_ids]
         usable_gems.sort(key=lambda x: x.get("requiredLevel", 0), reverse=True)  # Sort by required level descending
         await split_send_message(ctx,', '.join(gem['name'] for gem in usable_gems))
 
     if "!gear" in message:
-        # Get all gear items in item_ids
-        gear = [item for item in Items.get_gear_items() if item["id"] in item_ids]
-        progressive_message = build_progressive_message(gear)
-        singles = ', '.join(gear['name'] for gear in gear if "Progressive" not in gear["category"])
-        await split_send_message(ctx, progressive_message + (', and ' + singles if singles else '' ))
+        await split_send_message(ctx, generate_equipment_message(received_item_ids, total_ids=[item["id"] for item in Items.get_gear_items()]))
+
+    if "!weapons" in message:
+        await split_send_message(ctx, generate_equipment_message(received_item_ids, total_ids=[item["id"] for item in Items.get_weapon_items()]))
+
+    if any(cmd in message for cmd in ("!armor", "!armour")):
+        await split_send_message(ctx, generate_equipment_message(received_item_ids, total_ids=[item["id"] for item in Items.get_armor_items()]))
 
     if "!links" in message:
         # Get all linked items in item_ids
-        links = [item for item in Items.get_max_links_items() if item["id"] in item_ids]
+        links = [item for item in Items.get_max_links_items() if item["id"] in received_item_ids]
         link_counts: dict[str, int] = {}
         
         # Count each occurrence of each link item
-        for item_id in item_ids:
+        for item_id in received_item_ids:
             # Find the link item with this ID
             link_item = next((item for item in Items.get_max_links_items() if item["id"] == item_id), None)
             if link_item:
@@ -171,12 +174,12 @@ async def chat_commands_callback(ctx: "PathOfExileContext", line: str):
         
         await split_send_message(ctx, link_message)
 
-    if "!flasks" in message or "!flask" in message:
+    if any(cmd in message for cmd in ("!flasks", "!flask")):
         # Get all flask items in item_ids
         flask_counts: dict[str, int] = {}
         
         # Count each occurrence of each flask item
-        for item_id in item_ids:
+        for item_id in received_item_ids:
             # Find the flask item with this ID
             flask_item = next((item for item in Items.get_flask_items() if item["id"] == item_id), None)
             if flask_item:
@@ -190,26 +193,12 @@ async def chat_commands_callback(ctx: "PathOfExileContext", line: str):
 
         await split_send_message(ctx, flask_message)
 
-    if "!ascendancy" in message:
+    if any(cmd in message for cmd in ("!ascendancy", "!ascendancies", "!classes", "!class")):
         # Get all ascendancy items in item_ids
-        ascendancy = [item for item in Items.get_ascendancy_items() if item["id"] in item_ids]
+        ascendancy = [item for item in Items.get_ascendancy_items() if item["id"] in received_item_ids]
         await split_send_message(ctx,', '.join(ascendancy['name'] for ascendancy in ascendancy))
 
-    if "!weapons" in message:
-        # Get all weapon items in item_ids
-        weapons = [item for item in Items.get_weapon_items() if item["id"] in item_ids]
-        progressive_message = build_progressive_message(weapons)
-        singles = ', '.join(weapons['name'] for weapons in weapons if "Progressive" not in weapons["category"])
-        await split_send_message(ctx, progressive_message + (', and ' + singles if singles else '' ))
-
-    if "!armor" in message:
-        # Get all armor items in item_ids
-        armor = [item for item in Items.get_armor_items() if item["id"] in item_ids]
-        progressive_message = build_progressive_message(armor)
-        singles = ', '.join(armor['name'] for armor in armor if "Progressive" not in armor["category"])
-        await split_send_message(ctx, progressive_message + (', and ' + singles if singles else '' ))
-
-    if "!passive" in message or "!p" in message:
+    if any(cmd in message for cmd in ("!p", "!passive", "!passives")):
         message = f"You have {ctx.passives_available - ctx.passives_used} passive skill points available. ( {ctx.passives_used} / {ctx.passives_available} total for character {ctx.character_name} )"
         await split_send_message(ctx, message)
     
@@ -218,20 +207,115 @@ async def chat_commands_callback(ctx: "PathOfExileContext", line: str):
         deathlinked = ctx.get_is_death_linked()
         await ctx.update_death_link(not deathlinked)
         await split_send_message(ctx, f"Deathlink {"enabled" if not deathlinked else "disabled"}.")
+
+    if any (cmd in message for cmd in ("!whisper updates", "!whisper update", "!updates", "!update")):
+        ctx.whisper_updates_enabled = not ctx.whisper_updates_enabled
+        await split_send_message(ctx, f"Whisper updates are {"enabled" if not ctx.whisper_updates_enabled else "disabled"}.")
+        await ctx.update_settings()
         
     if "!goal" in message:
         await send_goal_message(ctx)
 
-def build_progressive_message(items: list["ItemDict"]) -> str:
-    progressive_count: dict[str, int] = {}
+
+def generate_equipment_message(received_item_ids, total_ids=None) -> str:
+    received_gear_ids = [item_id for item_id in received_item_ids if item_id in total_ids]
+    received_gear_items = [item for item in Items.get_gear_items() if item["id"] in received_gear_ids]
+    
+    # Get all gear items that match the total_ids filter
+    all_gear_items = [item for item in Items.get_gear_items() if item["id"] in total_ids]
+    
+    progressive_message = build_progressive_message(received_gear_ids, received_gear_items)
+    singles_message = build_singles_message(received_gear_ids, received_gear_items, all_gear_items)
+    return_message = ""
+    if progressive_message and singles_message:
+        return_message = ", and also: ".join([progressive_message, singles_message])
+    else:
+        return_message = progressive_message + singles_message
+    return return_message
+
+def build_progressive_message(items_ids: list[int], items: list[Items.ItemDict]) -> str:
+    item_count: dict[str, int] = {}
     progressive_message = ""
-    for item in items:
-        if "Progressive" in item["category"]:
-            progressive_count[item["name"]] = progressive_count.get(item["name"], 0) + 1
-    for name, count in progressive_count.items():
+    progressive_items = [item for item in items if "Progressive" in item["category"]]
+    for item_id in items_ids:
+        for item in progressive_items:
+            if item["id"] == item_id:
+                item_count[item["name"]] = item_count.get(item["name"], 0) + 1
+    for name, count in item_count.items():
         progressive_message += f"{rarity_from_progressive_count(count)} {name.replace("Progressive ", "")}, "
     return progressive_message.rstrip(", ")
 
+def build_singles_message(item_ids: list[int], items: list[Items.ItemDict], all_items: list[Items.ItemDict]) -> str:
+    # Filter out progressive items
+    non_progressive_items = [item for item in items if "Progressive" not in item["category"]]
+    
+    # Group items by rarity (Normal, Magic, Rare, Unique)
+    rarities = ["Normal", "Magic", "Rare", "Unique"]
+    received_by_rarity = {rarity: [] for rarity in rarities}
+    all_items_by_rarity = {rarity: [] for rarity in rarities}
+    
+    # Group received items by rarity
+    for item in non_progressive_items:
+        if item['id'] in item_ids:
+            for rarity in rarities:
+                if rarity in item["category"]:
+                    received_by_rarity[rarity].append(item)
+                    break
+    
+    # Group all available items by rarity (from total_ids)
+    for item in all_items:
+        if "Progressive" not in item["category"]:
+            for rarity in rarities:
+                if rarity in item["category"]:
+                    all_items_by_rarity[rarity].append(item)
+                    break
+    
+    # Check which rarities are complete
+    complete_rarities = []
+    for rarity in rarities:
+        all_ids = {item["id"] for item in all_items_by_rarity[rarity]}
+        received_ids = {item["id"] for item in received_by_rarity[rarity]}
+        if all_ids and all_ids == received_ids:  # Has all items of this rarity
+            complete_rarities.append(rarity)
+    
+    # Build message based on complete rarities
+    message_parts = []
+    
+    # Determine the highest complete rarity tier
+    if complete_rarities:
+        # Check for cumulative completion (need all lower tiers too)
+        if "Unique" in complete_rarities and "Rare" in complete_rarities and "Magic" in complete_rarities and "Normal" in complete_rarities:
+            message_parts.append("Any Rarity")
+        elif "Rare" in complete_rarities and "Magic" in complete_rarities and "Normal" in complete_rarities:
+            message_parts.append("Any up to Rare Rarity")
+        elif "Magic" in complete_rarities and "Normal" in complete_rarities:
+            message_parts.append("Any up to Magic Rarity")
+        elif "Normal" in complete_rarities:
+            message_parts.append("Any Normal Rarity")
+        
+        # Now add incomplete rarities' items individually
+        # Determine which rarities to show individually
+        if "Any Rarity" in message_parts:
+            incomplete_rarities = []  # Don't show anything else
+        elif "Any up to Rare Rarity" in message_parts:
+            incomplete_rarities = ["Unique"]
+        elif "Any up to Magic Rarity" in message_parts:
+            incomplete_rarities = ["Rare", "Unique"]
+        elif "Any Normal Rarity" in message_parts:
+            incomplete_rarities = ["Magic", "Rare", "Unique"]
+        else:
+            incomplete_rarities = rarities  # Show all
+        
+        for rarity in incomplete_rarities:
+            if rarity in received_by_rarity and received_by_rarity[rarity]:
+                message_parts.extend([item['name'] for item in received_by_rarity[rarity]])
+    else:
+        # No complete rarities, list all items individually
+        message_parts = [item['name'] for item in non_progressive_items if item['id'] in item_ids]
+    
+    return ', '.join(message_parts)
+
+
 def rarity_from_progressive_count(count: int) -> str:
     if count == 1:
         return "Normal"
@@ -299,19 +383,20 @@ async def send_goal_message(ctx):
     await split_send_message(ctx=ctx, message=goal_message)
 
 
-async def split_send_message(ctx, message: str, max_length: int = 500):
+async def split_send_message(ctx, message: str, max_length: int = 500, whisper_player: bool = True):
     """
     Splits a message into chunks and sends each chunk separately.
     """
     if not message:
         message = "None"
-    prefix = f"@{ctx.character_name} "
-    max_length = max_length - len(prefix)  # Adjust for character name prefix
+    prefix = ""
+    if whisper_player:
+        prefix = f"@{ctx.character_name} "
+        max_length = max_length - len(prefix)  # Adjust for character name prefix
     if len(message) <= max_length:
         await asyncio.wait_for(inputHelper.send_poe_text(prefix + message), SEND_MESSAGE_TIMEOUT)
         return
 
-    # Split the message into chunks, only at a comma
     chunks = [message[i:i + max_length] for i in range(0, len(message), max_length)]
     
     # Send each chunk
