@@ -40,7 +40,7 @@ last_zone = None
 # Timeouts (seconds)
 TIMEOUT = 5.0
 async def when_enter_new_zone(ctx: "PathOfExileContext", line: str):
-    global last_zone, is_char_in_logic
+    global last_zone
     zone = textUpdate.get_zone_from_line(ctx, line)
     last_zone = zone
     if not zone:
@@ -50,8 +50,8 @@ async def when_enter_new_zone(ctx: "PathOfExileContext", line: str):
     char = None
     found_items_list: list[Locations.LocationDict] = []
     if ctx.character_name is None or ctx.character_name == "":
-        logger.info("Character name is not set, cannot validate.")
-        await asyncio.wait_for(send_multiple_poe_text([f"/itemfilter {itemFilter.INVALID_FILTER_NAME}", "Character name is not set, cannot validate."]), TIMEOUT)
+        ctx.text_to_send.append(("Character name is not yet set! type `!ap char` to set your char.", True))
+        #await asyncio.wait_for(send_multiple_poe_text([f"/itemfilter {itemFilter.INVALID_FILTER_NAME}", "Character name is not set, cannot validate."]), TIMEOUT)
         return
     try:
         char = (await asyncio.wait_for(gggAPI.get_character(ctx.character_name),TIMEOUT)).character
@@ -63,20 +63,27 @@ async def when_enter_new_zone(ctx: "PathOfExileContext", line: str):
         logger.error(f"Error fetching character {ctx.character_name}: {e}\nTraceback:\n{tb_str}")
         raise
 
-    logic_errors = await validate_and_update(ctx, char, found_items_list) # this updates the filter if needed.
+    logic_errors = await validate_and_update(ctx, char, found_items_list) # this also updates the filter if needed.
+    char_in_logic = True if len(logic_errors) == 0 else False
     victory_task = check_for_victory(ctx, zone, char)
 
     # THIS IS FOR DEBUGGING PURPOSES, I'm tired of respeccing my character to test the logic, lol
     if False:
-        is_char_in_logic = True
+        char_in_logic = True
 
-    if not is_char_in_logic:
+    if not char_in_logic:
         error_msg = ", and ".join(logic_errors) if logic_errors else "unknown errors"
-        await asyncio.wait_for(send_multiple_poe_text([f"/itemfilter {itemFilter.INVALID_FILTER_NAME}", f"@{ctx.character_name} {INVALID_STATE_CHAT_ERROR_MESSAGE}: {error_msg}"]), TIMEOUT)
+        ctx.text_to_send.append((f"/itemfilter {itemFilter.INVALID_FILTER_NAME}", False))
+        ctx.text_to_send.append((f"{INVALID_STATE_CHAT_ERROR_MESSAGE}: {error_msg}", True))
+        #await asyncio.wait_for(send_multiple_poe_text([f"/itemfilter {itemFilter.INVALID_FILTER_NAME}", f"@{ctx.character_name} {INVALID_STATE_CHAT_ERROR_MESSAGE}: {error_msg}"]), TIMEOUT)
+        return
+    
     elif victory_task:
-        pass # callback handles victory and chat sending
-    else:
-        await asyncio.wait_for(inputHelper.important_send_poe_text(f"/itemfilter {itemFilter.AP_FILTER_NAME}", retry_times=9, retry_delay=0.5), TIMEOUT)
+        return # callback handles victory and chat sending
+
+    else: # valid character, not victorious yet
+        ctx.text_to_send.append((f"/itemfilter {itemFilter.AP_FILTER_NAME}", False))
+        #await asyncio.wait_for(inputHelper.important_send_poe_text(f"/itemfilter {itemFilter.AP_FILTER_NAME}", retry_times=9, retry_delay=0.5), TIMEOUT)
 
 def check_for_victory(ctx: "PathOfExileContext", zone: str, char: gggAPI.Character) -> asyncio.Task | None:
     goal = ctx.game_options.get("goal", -1)
@@ -113,7 +120,7 @@ def check_for_victory(ctx: "PathOfExileContext", zone: str, char: gggAPI.Charact
         for i in boss_set:
             bosses.append(Locations.bosses_by_id[i].get("name", "Unknown Boss"))
         if bosses:
-            await asyncio.create_task(inputHelper.send_poe_text(f"@{ctx.character_name} You have defeated {", ".join(bosses)}!", retry_times=40, retry_delay=0.5))
+            await asyncio.create_task(inputHelper.send_poe_text(f"@{ctx.character_name} You have completed {", ".join(bosses)}!", retry_times=40, retry_delay=0.5))
     # we should probably create a location and fill it with a goal item for each boss.
     if goal == Options.Goal.option_defeat_bosses:
         held_items = get_held_item_names_ilvls_from_char(char)
@@ -124,10 +131,11 @@ def check_for_victory(ctx: "PathOfExileContext", zone: str, char: gggAPI.Charact
             if not boss_data:
                 logger.error(f"Boss {boss} not found in Locations.bosses.")
                 raise ValueError(f"Boss {boss} not found in Locations.bosses.")
-            drops = boss_data.get("drops", [])
+            boss_drops = boss_data.get("drops", [])
             for item, ilvl in held_items:
-                if item in [i["name"] for i in drops]:
-                    if ilvl and drops[item].get("ilvl", 0) and ilvl < drops[item]["ilvl"]:
+                held_boss_drops = [i for i in boss_drops if i["name"] == item]
+                for boss_drop in held_boss_drops:
+                    if ilvl and boss_drop.get("ilvl", False) and ilvl < boss_drops[item]["ilvl"]:
                         continue # item is not high enough level
                     logger.info(f"Found goal item {item} in {zone}.")
                     boss_sent.append(boss)
@@ -251,7 +259,11 @@ def validate_char_equipment(character: gggAPI.Character, ctx: "PathOfExileContex
         errors.append(f"Class {character.class_}")
 
     gucci_rarity_check = {}
+    ignore_item_inventory_ids = ["BrequelGrafts","BrequelGrafts2"]
     for equipped_item in character.equipment:
+        if equipped_item.inventoryId in ignore_item_inventory_ids:
+            continue # ignore brequel grafts.
+
         rarity = equipped_item.rarity
         gucci_rarity_check.setdefault(rarity, 0)
         gucci_rarity_check[rarity] += 1
